#include <gsl/gsl_histogram.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_randist.h>

#include "gsl_distfit_core.h"


int main (void) {
  gsl_histogram *h;
  gsl_rng       *r;
  double        *s,*x,*y;
  double        sigma  = 1.0;
  double        mu     = 0.0;
  double R;
  size_t        bins   = 100;
  size_t        sample = 250000;
  size_t i;
  FILE *stream;

  /* Special variables for the fitting */
  gsl_vector             *fit_state;  
  gsl_distfit_ftype      *F;
  gsl_multifit_fdfsolver *solver;



  /* Starting with allocation ..*/
  r = gsl_rng_alloc (gsl_rng_mt19937);
  gsl_rng_set(r , 0);
  h = gsl_histogram_alloc(bins);
  gsl_histogram_set_ranges_uniform(h,-3*sigma,3*sigma);
  
  for (i=0; i < sample; i++) {
    R = mu + gsl_ran_gaussian(r,sigma);
    gsl_histogram_increment(h , R);
  }
  gsl_histogram_normalize(h);

  /* Don't have any reasonable values for uncertainities*/
  s = (double *) (calloc(bins,sizeof(double)));
  for (i=0; i<bins; i++)
    s[i] = 1.0;
  
  y = h->bin;
  x = gsl_distfit_xdata_alloc(h); 
  F = gsl_distfit_func("gaussian");
  /* 
     Nearly all subsequent calls to *_distfit_* functions take this
     F - as an argument.
  */
  
  fit_state = gsl_vector_alloc(gsl_distfit_psize(F));
  gsl_distfit_init(bins,x,y,F,fit_state);
  printf("Starting values: mu = %10.7f  sigma = %10.7f \n",gsl_vector_get(fit_state,0),gsl_vector_get(fit_state,1));
  solver    = gsl_distfit_solver_alloc(bins,F);
  gsl_distfit(solver,bins,x,y,s,F,fit_state);
  printf("Fit resulted in: mu = %10.7f  sigma = %10.7f \n",gsl_vector_get(fit_state,0),gsl_vector_get(fit_state,1));

  stream = fopen("RDist","w");
  gsl_distfit_fprintf(bins,x,y,NULL,F,fit_state,stream);
  fclose(stream);
  
  
  free(x);
  free(s);
  gsl_histogram_free(h);
  gsl_rng_free(r);
  gsl_multifit_fdfsolver_free(solver);
  gsl_distfit_ftype_free(F);
  return 1;
}
